﻿/*	Clase configuracion.GestorAccesibilidad
	Esta clase realiza la extraccion de datos de un archivo XML y establece un entorno para el comienzo
	de la actividad.
*/

import configuracion.xmlcnice.Xml2Object;
import configuracion.GestorTeclado;
import configuracion.GestorRaton; 
import configuracion.GestorSincro;
import configuracion.GestorBarridos;
import configuracion.GestorGrupos;
import configuracion.GestorConfiguracion;
import GestorInfoExterna;

class configuracion.GestorAccesibilidad
{
	//Atributos de la clase
	private var _debug:Boolean = false;
	//alTerminar se ejecuta cuando se ha completado la lectura y clasificacion del archivo de configuracion
	public var alTerminar:Function;
	public var metadatos:Object;
	//general contiene las opciones de configuracion
	public var general:Object;
	public var idiomasDisponibles:Array;
	//idiomasLabels se usa para mostrar los nombres de idiomas en la configuracion inicial
	public var idiomasLabels:Array;
	public var idioma:Array;
	//objetosAccesibles contiene la configuracion predeterminada de los objetos accesibles
	private var objetosAccesibles:Array;
	//arraySubtituloBruto contiene los subtitulos de la actividad como identificadores
	private var arraySubtituloBruto:Array;
	public var mascara:Object;
	//Refuerzos
	public var r_positivos:Array;
	public var r_negativos:Array;
	//myXML para el XML de configuracion
	private var myXML:Xml2Object;
	//myXML2 para el XML de idioma
	private var myXML2:Xml2Object;

	//Gestores integrados
	private var gEvAct:Object;
	public var gTeclado:GestorTeclado;
	public var gRaton:GestorRaton;
	public var gSincro:GestorSincro;
	public var gBarridos:GestorBarridos;
	public var gGrupos:GestorGrupos;
	public var gConfig:GestorConfiguracion;
	public var gInfoExterna:GestorInfoExterna;

	private var _ruta:String;
	//Prueba de locutor
	private var _locutor:Object

	/*	GestorAccesibilidad(ruta, config, llamante)
		(1) ruta contiene un string vacio si se carga una secuencia y "../" si se carga una actividad, segun
		nuestra jerarquia de directorios.
		(2) config contiene la direccion relativa donde se encuentra el archivo XML de configuracion.
		(3) llamante sera el gestor de eventos que se quiere asociar, tipicamente "this" desde la actividad.
		Pre: ninguno de los parametros de la funcion deben ser undefined.
		Post: inicializa los atributos de la clase y carga el XML de configuracion.
	*/
	function GestorAccesibilidad(ruta:String, config:String, llamante:Object)
	{
		//Inicializo los atributos
		myXML = new Xml2Object();
		myXML2 = new Xml2Object();
		metadatos = new Object();
		general = new Object();
		idiomasDisponibles = new Array();
		idiomasLabels = new Array();
		idioma = new Array();
		objetosAccesibles = new Array();

		r_positivos = new Array();
		r_negativos = new Array();

		//gEvAct deberia ser del tipo Notificacion, de esta forma implementariamos la funcion Notificar de las
		//actividades como un metodo de la clase y no como una funcion anonima.
		gEvAct = llamante;
		gTeclado = new GestorTeclado(gEvAct);
		gRaton = new GestorRaton(gEvAct);
		gSincro = new GestorSincro(this);
		gGrupos = new GestorGrupos(gEvAct, this);
		gBarridos = new GestorBarridos(gGrupos);
		gInfoExterna = new GestorInfoExterna();
		gConfig = new GestorConfiguracion(this, gInfoExterna);

		establecerRuta(ruta);

		//Configuro la accion a realizar al completar la lectura del XML
		myXML.onLoad = function()
		{
			//Ahora extraere las distintas informaciones que me interesen
			if(_debug) trace("    Actividad: " + this.datum.attributes.nombre);
			this._referrer.categorizarDatos.call(this._referrer, this.datum);
			//Lanzo la ventanas de configuracion y teclado
			this._referrer.configuracionManual.call(this._referrer);
			//Invoco la funcion alTerminarXml
			this._referrer._alTerminarXml.call(this._referrer);
		}
		//Para terminar, lanzo la lectura del archivo de configuracion, pasandole una referencia a mi mismo.
		if(_debug) trace("*** Cargando XML de configuracion: " + config);
		myXML.load(config, this);
	}

	//categorizarDatos(nodosXML)
	private function categorizarDatos(nodosXML:Array)
	{
		var i:Number;
		this.metadatos = nodosXML.attributes;
		for (i = 0; i < nodosXML.length; i++)
		{
			switch (nodosXML[i].tipo)
			{
				case "configuracionGeneral":
					if(_debug) trace("*** Categorizando la configuracion general...");
					general = nodosXML[i];
					gConfig.SetConfiguracion(general);
					break;
				case "idiomas":
					if(_debug) trace("*** Categorizando idiomas...");
					for (var j:Number = 0; j < nodosXML[i].length; j++)
					{
						idiomasDisponibles[nodosXML[i][j].nombre] = nodosXML[i][j].archivo;
						idiomasLabels[j] = nodosXML[i][j].nombre;
						if(_debug) trace("    Encontrado idioma: "+ nodosXML[i][j].nombre);
					}
					break;
				case "objetosAccesibles":
					if(_debug) trace("*** Categorizando objetos accesibles...");
					objetosAccesibles = nodosXML[i];
					break;
				case "subtitulos":
					if(_debug) trace("*** Categorizando subtitulos...");
					arraySubtituloBruto = nodosXML[i];
					break;
				case "refuerzos":
					if(_debug) trace("*** Categorizando refuerzos...");
					var j:Number;
					for (j = 0; j < nodosXML[i].length; j++)
					{
						//Meto el objeto en el array que me interesa.
						if (nodosXML[i][j].tipo == "refuerzo_positivo")
						{
							r_positivos[r_positivos.length] = nodosXML[i][j];
							if(_debug) trace("Encontrado refuerzo positivo: " +
											 r_positivos[r_positivos.length-1].id_texto);
						}
						if (nodosXML[i][j].tipo == "refuerzo_negativo")
						{
							r_negativos[r_negativos.length] = nodosXML[i][j];
							if(_debug) trace("Encontrado refuerzo negativo: " +
											 r_negativos[r_negativos.length-1].id_texto);
						}
					}
					break;
				default:
					if(_debug) trace("Datos no categorizados.");
			}
		}
	}

	//extraerIdioma(obj)
	private function extraerIdioma(obj:Array)
	{
		if(_debug) trace("*** Extrayendo datos...");
		var i:Number;
		idioma.url_audios = _ruta + obj.attributes["url_audios"];
		idioma.url_audios_generales = _ruta + obj.attributes["url_audios_generales"];
		if(_debug) trace("    URL: "+ idioma.url_audios);
		for (i = 0; i < obj.length; i++)
		{
			switch (obj[i].tipo)
			{
				case "cadena":
					idioma[obj[i].id_texto] = obj[i];
					if(_debug) trace("    Cadena: "+ traducir(obj[i].id_texto));
					break;
				default:
					if(_debug) trace("Datos no categorizados");
			}
		}
	}

	//_aplicarAccesibilidad()
	private function _aplicarAccesibilidad()
	{
		if(_debug) trace("*** Aplicando Accesibilidad...");
		//Para cada elemento de objetosAccesibles establezco sus parametros de accesibilidad
		var i:Number;
		//He añadido un if para poder poner un tag en los xml si no queremos que registre algun objeto como
		//accesible
		var objeto_aux: Object;
		var cadena : String;
		for(i=0; i < objetosAccesibles.length; i++)
		{
			objeto_aux = objetosAccesibles[i];
			cadena = objeto_aux.no_es_movieclip;
			if (cadena == "true")
			{	// esto lo hacemos para crear un nuevo objeto en la lista, sino lo sacara undefined
				gEvAct[objetosAccesibles[i].objeto]=new Object;
				aplicarAccesibilidad(objetosAccesibles[i]);
			}
			else
			{
				aplicarAccesibilidad(objetosAccesibles[i]);
				gBarridos.registrar(gEvAct[objetosAccesibles[i].objeto], objetosAccesibles[i].tabIndex);
				gGrupos.insertar(gEvAct[objetosAccesibles[i].objeto], objetosAccesibles[i].grupo);
			}
		}
	}

	//aplicarAccesibilidad(obj_acc)
	private function aplicarAccesibilidad(obj_acc:Object)
	{
		if (gEvAct[obj_acc.objeto]._accProps == undefined)
		{
			gEvAct[obj_acc.objeto]._accProps = new Object();
		}
		if (obj_acc.tabIndex != undefined)
		{
			gEvAct[obj_acc.objeto].tabIndex = obj_acc.tabIndex;
		}
		if (traducir(obj_acc.id_etiqueta) != undefined)
		{
			gEvAct[obj_acc.objeto]._accProps.name = traducir(obj_acc.id_etiqueta);
		}
		if (traducir(obj_acc.id_descripcion) != undefined)
		{
			gEvAct[obj_acc.objeto]._accProps.description = traducir(obj_acc.id_descripcion);
		}
		if (darAudio(obj_acc.id_etiqueta) != undefined)
		{
			gEvAct[obj_acc.objeto]._accProps.audio = idioma.url_audios + darAudio(obj_acc.id_etiqueta);
		}
		if (darAudioGeneral(obj_acc.id_etiqueta) != undefined)
		{
			gEvAct[obj_acc.objeto]._accProps.audio = idioma.url_audios_generales +
													 darAudioGeneral(obj_acc.id_etiqueta);
		}
		if ((obj_acc.shortcut != undefined ) and
			(gTeclado.Registrar(obj_acc.shortcut,gEvAct[obj_acc.objeto])==-1))
		{
			if(_debug) trace("Error: combinacion no disponible"+ obj_acc.shortcut );
		}
		if(_debug) trace("*** " + gEvAct[obj_acc.objeto] + newline +
			"    TabIndex: " + gEvAct[obj_acc.objeto].tabIndex + newline +
			"    Nombre corto: " + gEvAct[obj_acc.objeto]._accProps.name + newline +
			"    Descripcion: " + gEvAct[obj_acc.objeto]._accProps.description + newline +
			"    Shortcut: " + gEvAct[obj_acc.objeto]._accProps.shortcut + newline +
			"    Audio: " + gEvAct[obj_acc.objeto]._accProps.audio);
		//Para cada elemento accesible lo registro en gRaton
		gRaton.Registrar(gEvAct[obj_acc.objeto]);
	}

	/*	copiarDatosAClip(origen, destino)
		Este metodo lo utilizaremos cuando no queramos registrar un objeto accesible como tal, mediante el tag
		<param name="no_es_movieclip">true</param> y posteriormente queramos asignarle su informacion a otro
		objeto si registrado como accesible. Recibe dos parametros:
		(1) El objeto accesible origen.
		(2) El objeto accesible destino.
		Pre: ninguno de los parametros de la funcion deben ser undefined.
		Post: copia la informacion del objeto accesible origen al destino.
	*/
	public function copiarDatosAClip(origen:String, destino:String)
	{
		var objeto0_aux:Object;
		var objeto1_aux:Object;
		var objeto2_aux:Object;
		var i:Number;
		for(i=0; i < objetosAccesibles.length; i++)
		{
			objeto0_aux = objetosAccesibles[i];
			if (objeto0_aux.objeto == origen){
				if(_debug) trace("Encontrado origen: " + objeto0_aux.objeto)
				objeto1_aux = objeto0_aux;
				}
			if (objeto0_aux.objeto == destino){
				if(_debug) trace("Encontrado destino: " + objeto0_aux.objeto)
				objeto2_aux = objeto0_aux;
				}
		}
		objeto2_aux.id_etiqueta = objeto1_aux.id_etiqueta;
		gEvAct[objeto2_aux.objeto]._accProps.name = gEvAct[objeto1_aux.objeto]._accProps.name;
		gEvAct[objeto2_aux.objeto]._accProps.description = gEvAct[objeto1_aux.objeto]._accProps.description;
		gEvAct[objeto2_aux.objeto]._accProps.audio = gEvAct[objeto1_aux.objeto]._accProps.audio;
		for(i=0; i < objetosAccesibles.length; i++)
		{
			objeto0_aux = objetosAccesibles[i];
			if (objeto0_aux.objeto == destino){
				if(_debug) trace("+++++++++ Copiando a no registrado +++++++");
				if(_debug) trace("Destino definitivo: " + objeto0_aux.objeto)
				objetosAccesibles[i] = objeto2_aux;
				}
		}
		if(_debug) trace("*** " + gEvAct[objeto2_aux.objeto] + newline +
			"    TabIndex: " + gEvAct[objeto2_aux.objeto].tabIndex + newline +
			"    Nombre corto: " + gEvAct[objeto2_aux.objeto]._accProps.name + newline +
			"    Descripcion: " + gEvAct[objeto2_aux.objeto]._accProps.description + newline +
			"    Shortcut: " + gEvAct[objeto2_aux.objeto]._accProps.shortcut + newline +
			"    Audio: " + gEvAct[objeto2_aux.objeto]._accProps.audio);
	}

	/*	darDescripcion(clip)
		(1) clip contiene el nombre de un objeto accesible.
		Pre: el string que recibe el metodo no debe ser undefined.
		Post: devuelve la descripcion de un determinado objeto accesible.
	*/
	public function darDescripcion(clip:String):String{
		if(_debug) trace("Descripcion asignada a "+clip+": "+ gEvAct[clip]._accProps.description);
		return gEvAct[clip]._accProps.description;
	}

	/*	darNombreCorto(clip)
		(1) clip contiene el nombre de un objeto accesible.
		Pre: el string que recibe el metodo no debe ser undefined.
		Post: devuelve el nombre de un determinado objeto accesible.
	*/
	public function darNombreCorto(clip:String):String{
		if(_debug) trace("Nombre corto asignado a "+clip+": "+ gEvAct[clip]._accProps.name);
		return gEvAct[clip]._accProps.name;
	}

	/*	darTabIndex(clip)
		(1) clip contiene el nombre de un objeto accesible.
		Pre: el string que recibe el metodo no debe ser undefined.
		Post: devuelve el tabIndex de un determinado objeto accesible.
	*/
	public function darTabIndex(clip:String):Number{
		return gEvAct[clip].tabIndex;
	}

	//extraerSubtitulos(datos)
	private function extraerSubtitulos(datos:Array)
	{
		if(_debug) trace("*** Procesando Subtitulos...");
		//Array de arrays (agrupa subtitulos por escenas)
		gSincro.subtitulos = new Array();
		gSincro.audios = new Array();
		//Array que indica por donde va el indice de subtitulos de una escena
		var indexes = new Array();
		for (var i=0;i<arraySubtituloBruto.length;i++)
		{
			if(_debug) trace("    [" + arraySubtituloBruto[i].escena + "] " +
							 traducir(arraySubtituloBruto[i].id_texto));
			if(arraySubtituloBruto[i].tipo == "subtitulo")
			{
				var escena = arraySubtituloBruto[i].escena -1;
				if (indexes.length < (escena+1))
				{
					indexes[escena]=0;
					gSincro.subtitulos[escena] = new Array();
					gSincro.audios[escena] = new Array();
				}
				else
				{
					indexes[escena]++;
				}
				gSincro.subtitulos[escena][indexes[escena]] = traducir(arraySubtituloBruto[i].id_texto);
				if(_debug) trace("    [" + escena + "," + indexes[escena]+ "] " +
								 gSincro.subtitulos[escena][indexes[escena]]);
				//Y ahora meto las locuciones asociadas en gSincro.audios
				// HMC
				if ( idioma[arraySubtituloBruto[i].id_texto].audio_general )
				{
					gSincro.audios[escena][indexes[escena]] = idioma.url_audios_generales +
														  idioma[arraySubtituloBruto[i].id_texto].audio_general;
				}
				else
				{
					gSincro.audios[escena][indexes[escena]] = idioma.url_audios +
														  idioma[arraySubtituloBruto[i].id_texto].audio;
				}
				// HMC
				if(_debug) trace("    [" + escena + "," + indexes[escena]+ "] " +
								 gSincro.audios[escena][indexes[escena]]);
			}
		}
	}

	//_alTerminarXml()
	private function _alTerminarXml()
	{
		//Ahora cargo el archivo de idioma:
		this.myXML2.onLoad = function()
		{
			//Ahora extraere las distintas informaciones que me interesen
			this._referrer.extraerIdioma.call(this._referrer,this.datum);
			this._referrer.extraerSubtitulos.call(this._referrer,this.datum);
			this._referrer._aplicarAccesibilidad.call(this._referrer);
			this._referrer._alTerminarXml2.call(this._referrer);
		}
		this.myXML2.load(_ruta + idiomasDisponibles[gConfig.GetIdioma()], this);
		if(_debug) trace("*** Cargando XML de idioma:" + idiomasDisponibles[gConfig.GetIdioma()]);		
		delete this.myXML;
	}

	/*	establecerRuta(ruta)
		(1) ruta contiene un string vacio si se carga una secuencia y "../" si se carga una actividad, segun
		nuestra jerarquia de directorios.
		Pre: el string que recibe el metodo no debe ser undefined.
		Post: establece el valor del atributo _ruta.
	*/
	public function establecerRuta(ruta:String)
	{
		_ruta = ruta;
	}

	/*	obtenerRuta()
		Pre: cierto.
		Post: devuelve el valor del atributo _ruta.
	*/
	public function obtenerRuta():String
	{
		return _ruta;
	}

	//_alTerminarXml2()
	private function _alTerminarXml2()
	{
		delete this.myXML2;
		//Inicializo el gestor de Subtitulacion
		gSincro.registrarTextBox(gEvAct[general.objSubtitulos],
								 gConfig.GetDuracionTextos(),
								 gEvAct[general.objRefuerzo]);
		if(general.ayudaContextual != undefined)
		{
			registrarLocutor(gEvAct[general.ayudaContextual]);
		}
		if(gConfig.GetModoNavegacion() == "B_AUTO")
		{
			barridoAutomatico(gConfig.GetIntervaloBarrido());
		}
		else
		{
			if(gConfig.GetModoNavegacion() == "B_MANU")
			{
				barridoAutomatico(-1); //Activo el barrido manual con pulsadores!!!
			}
			else
			{
				barridoAutomatico(0);
			}
			//AUTOTRADUCIR LOS BOTONES DE LA BARRA!
			if (traducir("txt_eval") != undefined)
			{
				gEvAct["txt_eval"] = traducir("txt_eval");
			}
			if (traducir("txt_salir") != undefined)
			{
				gEvAct["txt_salir"] = traducir("txt_salir");
			}
			if (traducir("txt_ayuda") != undefined)
			{
				gEvAct["txt_ayuda"] = traducir("txt_ayuda");
			}
			if (traducir("txt_volver") != undefined)
			{
				gEvAct["txt_retroceder"] = traducir("txt_volver");
			}
			if (traducir("txt_imprimir") != undefined)
			{
				gEvAct["txt_imprimir"] = traducir("txt_imprimir");
			}
		}
		this.alTerminar();
	}

	//configuracionManual()
	private function configuracionManual()
	{
		//Solo lanzo la ventana de configuracion si me lo han indicado en el XML.
		if (_level0.ventanaConfiguracion != undefined and _level0.ventanaTeclas != undefined
			and general.configurable.toLowerCase() != "no")
		{
			if(_debug) trace("Voy a enlazar las ventanas con gAcces");
			_level0.ventanaConfiguracion.cargar(this);
			_level0.ventanaTeclas.cargar(this);
			_level0.creditos.cargar(this);
		}
	}

	/*	registrarObjetosAccesibles(lista)
		Permite establecer a peticion las propiedades de accesibilidad para una lista de objetos dada.
		Pre: cierto.
		Post: registra una lista de objetos accesibles.
	*/
	public function registrarObjetosAccesibles(lista:Array)
	{
		var i:Number;
		if(_debug) trace("Registrando lista...");
		//Busco un objeto entre mis objetos accesibles y le aplico la accesibilidad
		for (i=0; i<lista.length; i++)
		{
			if(_debug) trace("registrando elemento " + lista[i]);
			registrarObjetoAccesible(lista[i])
		}
	}

	/*	registrarObjetosAccesibles(obj)
		Permite establecer a peticion las propiedades de accesibilidad para un objeto dado. Se recogen
		automaticamente todos los datos posibles del archivo XML de configuracion.
		Pre: cierto.
		Post: registra un objeto accesible.
	*/
	public function registrarObjetoAccesible(obj:Object)
	{
		if(_debug) trace("registrando " + obj);
		var i:Number;
		//Busco un objeto entre mis objetos accesibles y le aplico la accesibilidad
		for (i=0;i<objetosAccesibles.length;i++)
		{
			if(_debug) trace(gEvAct[objetosAccesibles[i].objeto] + " == " + obj + "?");
			if (gEvAct[objetosAccesibles[i].objeto] == obj)
			{
				aplicarAccesibilidad(objetosAccesibles[i]);
				gBarridos.registrar(gEvAct[objetosAccesibles[i].objeto], objetosAccesibles[i].tabIndex);
				gGrupos.insertar(gEvAct[objetosAccesibles[i].objeto], objetosAccesibles[i].grupo);
				return;
			}
		}
	}

	/*	registrar(combinacion, obj)
		(1) combinacion de teclado.
		(2) objeto accesible que registrar para raton y teclado.
		Pre: cierto.
		Post: dada una combinacion de teclado y su objeto correspondiente, lo registra para raton y para teclado.
	*/
	public function registrar(combinacion:String, obj:Object)
	{
		gRaton.Registrar(obj);
		gTeclado.Registrar(combinacion, obj);
	}

	/*	traducir(id)
		Pre: id es un identificador de texto.
		Post: devuelve su traduccion, segun el idioma actual.
	*/
	public function traducir(id:String):String
	{
		return idioma[id].texto;
	}

	/*	darAudio(id)
		Pre: id es un identificador de texto.
		Post: devuelve el nombre del archivo de audio.
	*/
	public function darAudio(id:String):String
	{
		return idioma[id].audio;
	}

	/*	darAudioGeneral(id)
		Pre: id es un identificador de texto.
		Post: devuelve el nombre del archivo de audio general.
	*/
	public function darAudioGeneral(id:String):String
	{
		return idioma[id].audio_general;
	}

	/*	darTiempo(id)
		Pre: id es un identificador de texto.
		Post: devuelve el tiempo que tarda en reproducirse.
	*/
	public function darTiempo(id:String):Number
	{
		return idioma[id].tiempo;
	}

	/*	darImagen(id)
		Pre: id es un identificador de texto.
		Post: devuelve la imagen asociada que debe instanciarse.
	*/
	public function darImagen(id:String):String
	{
		return idioma[id].imagen;
	}

	/*	darSwf(id)
		Pre: id es un identificador de texto.
		Post: devuelve el swf que hay que cargar si se selecciona.
	*/
	public function darSwf(id:String):String
	{
		return idioma[id].swf;
	}

	/*	darParamSwf(id)
		Pre: id es un identificador de texto.
		Post: devuelve los parametros del swf que se carga.
	*/
	public function darParamSwf(id:String):String
	{
		return idioma[id].paramswf;
	}

	/*	establecerIdioma(id_idioma)
		Pre: debe recibir un nombre de idioma de la lista idiomasDisponibles.
		Post: cambiar el idioma a utilizar en las traducciones de etiquetas identificadoras.
	*/
	public function establecerIdioma(id_idioma:String)
	{
		this.myXML2 = new Xml2Object();
		this.gConfig.SetIdioma(id_idioma);
		_alTerminarXml();
	}

	/*	registrarLocutor(locutor)
		Post: Permite indicar explicitamente que elemento de la actividad queremos que muestre las descripciones
		de botones disponibles.
	*/
	public function registrarLocutor(locutor:Object)
	{
		_locutor = locutor;
		for (var i=0;i<objetosAccesibles.length;i++)
		{
			if (gEvAct[objetosAccesibles[i].objeto]._accProps.description != undefined)
			{
				gEvAct[objetosAccesibles[i].objeto].onRollOver = function ()
				{
					locutor.text = this._accProps.description;
					if(_debug) trace(this._accProps.description);
				}
			}
		}
	}

	/*	bloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Detiene la captura de eventos, tanto de ratón como de teclado.
	*/
	public function bloquear(lista:Object, pI:Number, pF:Number)
	{
		gTeclado.Bloquear(lista, pI, pF);
		gRaton.Bloquear(lista, pI, pF);
		gBarridos.bloquear(lista, pI, pF);
		gBarridos.detenerse = true;
		gBarridos.detener();
		//Oculta el marcador de barrido
		gEvAct["marcador"].clear();
	}

	/*	desbloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Restaura la captura de eventos, tanto de ratón como de teclado.
	*/
	public function desbloquear(lista:Array, pI:Number, pF:Number)
	{
		gTeclado.Desbloquear(lista, pI, pF);
		gRaton.Desbloquear(lista, pI, pF);
		gBarridos.desbloquear(lista, pI, pF);
		if(gConfig.GetModoNavegacion() == "B_AUTO")
		{
			gBarridos.detenerse = false;
			gBarridos.resumir();
		}
	}

	/*	barridoAutomatico(tiempoBarrido)
		Pre: Toma como parametro el tiempo de barrido.
		Post: Arranca el modo de navegación por barrido automatico. No es necesario el uso de este metodo si
		ya se estableció el parametro general barrido_automatico desde el fichero XML de configuración.
	*/
	public function barridoAutomatico(tiempoBarrido:Number)
	{
		for (var i=0;i<objetosAccesibles.length;i++)
		{
			gEvAct[objetosAccesibles[i].objeto].onRollOver = function () {}
		}
		mascara = new Object();
		if(tiempoBarrido == -1)
		{
			//Modo de barrido de pulsadores
			mascara = gBarridos.barridoAutomatico(0, objetosAccesibles, gEvAct, true);
			gBarridos._tiempoBarrido = 0;
			gBarridos.detener();
			gRaton.Registrar(mascara);
			//Ahora activo el capturador de pulsaciones
			gBarridos.capturadorPulsador = new configuracion.CapturadorPulsador(gBarridos, true);
		}
		else
		{
			mascara = gBarridos.barridoAutomatico(tiempoBarrido, objetosAccesibles, gEvAct);
			if(gConfig.GetModoNavegacion() != "B_AUTO")
			{
				gBarridos._tiempoBarrido = 0;
				gBarridos.detener();
			}
			if ((tiempoBarrido != 0) and (gConfig.GetModoNavegacion() == "B_AUTO"))
			{
				if(_debug) trace("BARRIDO AUTOMATICO: " + mascara);
				gRaton.Registrar(mascara);
				//Ahora activo el capturador de pulsaciones
				gBarridos.capturadorPulsador = new configuracion.CapturadorPulsador(gBarridos, true);
			}
		}
		if(gConfig.GetTeclaSeleccion() != undefined){
			gTeclado.Registrar(gConfig.GetTeclaSeleccion(), mascara);
		}
		if(gConfig.GetTeclaAvanceBarrido() != undefined){
			gTeclado.Registrar(gConfig.GetTeclaAvanceBarrido(), gBarridos._avance);
		}
		if(gConfig.GetTeclaRetrocesoBarrido() != undefined){
			gTeclado.Registrar(gConfig.GetTeclaRetrocesoBarrido(), gBarridos._retroceso);
		}
		if(gConfig.GetTeclaAvanceGrupo() != undefined){
			gTeclado.Registrar(gConfig.GetTeclaAvanceGrupo(), gGrupos._maestro);
		}
		if(gConfig.GetTeclaRetrocesoGrupo() != undefined){
			gTeclado.Registrar(gConfig.GetTeclaRetrocesoGrupo(), gGrupos._retro_maestro);
		}
		if (general.nombre_barra != undefined){
			gGrupos.barra = general.nombre_barra;
		}
		else{
			gGrupos.barra = "barra";
		}
	}

	/*	limpiar()
		Post: libera ordenadamente los gestores de entrada/salida utilizados.
		Debería invocarse cada vez que abandonemos una actividad.
	*/
	public function limpiar()
	{
		if(_debug) trace("Limpiando GestorConfiguracion");
		//Limpia el gestor de sincronizacion
		gSincro.limpiar();
		//Libera el controlador de teclado
		gTeclado.LiberarTeclado();
		//Libera el controlador de raton
		gRaton.LiberarRaton();
		//Libera el controlador de barridos
		gBarridos.limpiar();
		Selection.removeListener(gGrupos);
	}

	/*	liberar(obj)
		Post: Quita las escuchas (teclado, ratón y barridos) para un objeto dado.
	*/
	public function liberar(obj:Object)
	{
		gTeclado.LiberarObjeto(obj);
		//Añadir aqui una llamada a liberarObjeto en Barridos automaticos
		gBarridos.liberarObjeto(obj);
	}

	/*	trazarTeclasBasicas()
		Post: Muestra un listado de las teclas actualmente configuradas para la actividad actual.
	*/
	public function trazarTeclasBasicas()
	{
		if(_debug) trace("Tecla       |       Accion");
		if(_debug) trace("--------------------------");
		if(_debug) trace("Barra espaciadora" + " | " + "Pulsar el objeto seleccionado");
		if (gConfig.GetTeclaSeleccion().toLowerCase() != "barra espaciadora")
		{
			if(_debug) trace(gConfig.GetTeclaSeleccion() + " | " + "Pulsar el objeto seleccionado");
		}
		if(_debug) trace(gConfig.GetTeclaAvanceBarrido() + " | " + "Seleccionar el siguiente objeto");
		if(_debug) trace(gConfig.GetTeclaRetrocesoBarrido() + " | " + "Seleccionar el objeto anterior");
		if(_debug) trace("Tabulador" + " | " + "Cambiar de grupo de seleccion");
	}
}
