﻿/*	Clase configuracion.GestorBarridos
	Esta clase gestiona el funcionamiento de los barridos y sus temporizadores.
	Utiliza la clase CapturadorPulsador.
*/

import configuracion.CapturadorPulsador;
import configuracion.GestorGrupos;

class configuracion.GestorBarridos
{
	private var _debug:Boolean = false;

	//Atributos de la clase
	public var _tiempoBarrido:Number=0;
	public var _pulsadores:Boolean = false;
	private var _intervalo:Number=0;
	public var _posicionFoco:Number;
	private var _ordenObjetos:Array;
	private var mc_btn:MovieClip;
	private var gEvAct:Object;
	private var _activado:Boolean = false;

	private var _alPulsarBackUp:Function;

	public var color_marcador:String;
	public var capturadorPulsador:CapturadorPulsador;

	public var _avance:Object;
	public var _retroceso:Object;
	public var detenerse:Boolean;

	private var _gGrupos:configuracion.GestorGrupos;
	private var _gAcces:configuracion.GestorAccesibilidad;

	private var _audioDescribir:Boolean;
	private var _permitirAudioDescribir:Boolean;
	private var _habilitarBarrido:Boolean;

	/*	GestorBarridos(gGrupos)
		Constructor de la clase.
		Pre: Recibe un gestor de grupos.
		Post: Inicializa los pulsadores de avance, retroceso y el gestor de barrido en general.
	*/
	public function GestorBarridos(gGrupos:GestorGrupos)
	{
		_gGrupos = gGrupos;
		_gAcces = _gGrupos.getPadre();
		_audioDescribir = _gAcces.gConfig.GetAudioDescripcion();
		_permitirAudioDescribir = false;
		_habilitarBarrido = false;
		_activado = false;
		var btRef = mc_btn;
		mc_btn = eval(btRef);
		//Pulsador de avance
		_avance = new Object();
		_avance.addProperty("gBarridos");
		_avance.gBarridos = this;
		_avance.alPulsar = function()
		{
			if(_debug) trace("PULSADO AVANCE");
			this.gBarridos.avanzar();
		};
		//Pulsador de retroceso
		_retroceso = new Object();
		_retroceso.addProperty("gBarridos");
		_retroceso.gBarridos = this;
		_retroceso.alPulsar = function()
		{
			if(_debug) trace("PULSADO RETROCESO");
			this.gBarridos.retroceder();
		};
	}

	/*	darPosicionFoco()
		Post: Devuelve la posicion del foco (numero).
	*/
	public function darPosicionFoco():Number
	{
		return this._posicionFoco;
	}

	/*	darMc_btn()
		Post: Devuelve el atributo _avance (objeto).
	*/
	public function darMc_btn():Object
	{
		return this._avance;
	}

	/*	barridoAutomatico(tiempoBarrido, objetos, gEvAct, pulsadores)
		(1) Tiempo de barrido automatico, en milisegundos.
		(2) Array de objetos del barrido.
		(3) Referencia al llamante, que servira para enviar las notificaciones.
		(4) Booleano que indica si hay barrido por pulsadores.
		Post: Devuelve el objeto que enmascara el barrido, para poder registrarlo en el teclado.
	*/
	public function barridoAutomatico(tiempoBarrido:Number, objetos:Array, gEvAct, pulsadores:Boolean):Object
	{
		clearInterval(_intervalo);
		this.gEvAct =gEvAct;
		if (pulsadores != undefined){
			_pulsadores = pulsadores;
		}
		_activado = false;
		if (tiempoBarrido == undefined){
			_tiempoBarrido = 2000;
		}
		else{
			if (tiempoBarrido < 1000){
				_tiempoBarrido = tiempoBarrido * 1000;
			}
			else{
				_tiempoBarrido = tiempoBarrido;
			}
		}
		_posicionFoco =0;
		//Establezco un array ordenado segun los tabIndex para todos los objetos registrados
		_ordenObjetos = new Array();
		for (var i=0;i<objetos.length;i++)
		{
			_ordenObjetos[gEvAct[objetos[i].objeto].tabIndex] = gEvAct[objetos[i].objeto];
			//Para cada objeto registrado añado un comportamiento para cuendo suelta el foco y cuando lo recoge.
			gEvAct[objetos[i].objeto].gBarridos = this;
			gEvAct[objetos[i].objeto].onKillFocus = function()
			{
				gEvAct["marcador"].clear();
			};
			gEvAct[objetos[i].objeto].onSetFocus = function()
			{
				if(_debug) trace("gEvAct: " + this.gBarridos.gEvAct);
				_level0.crearMarcador.call(this.gBarridos.gEvAct,this);
				this.gBarridos._posicionFoco = this.tabIndex;
			};
			gEvAct[objetos[i].objeto].tabEnabled = false;
		}
		//Muestro los parametros del selector
		if(_debug) trace("----------------------");
		if(_debug) trace("beginIndex: " + 0 + "(" + Selection.getBeginIndex() + ")");
		if(_debug) trace("endIndex: " + _ordenObjetos.length + "(" + Selection.getEndIndex() + ")");
		if(_debug) trace("Punto de insercion: " + Selection.getCaretIndex());
		if(_debug) trace("Focus: " + _ordenObjetos[0] + "(" + Selection.getFocus() + ")");
		if (tiempoBarrido == 0)
		{
			if(_debug) trace("Activado barrido manual");
			Selection.setFocus(_ordenObjetos[0]);
			//No quiero que se muestre la marca desde el principio
			gEvAct["marcador"].clear();
		}
		else
		{
			if(_intervalo == 0 and _habilitarBarrido){
				_intervalo = setInterval(this, "avanzar", this._tiempoBarrido);
			}
		}
		return(cubrirRaton(this));
	}

	/*	Funciones de audio-descripcion.
		(1) audioDescribir(activar): Activa o desactiva la audio-descripcion (a nivel de configuracion).
		(2) permitirAudioDescribir(permitir): Inhibe o desinhibe la audio-descripcion. Se utilizara para
		controlar dentro de una actividad cuando permitimos y no permitimos audio-describir, a pesar de que este
		activada la audio-descripcion en el gestor de configuracion.
	*/
	public function audioDescribir(activar:Boolean)
	{
		_audioDescribir = activar;
	}
	public function permitirAudioDescribir(permitir:Boolean)
	{
		_permitirAudioDescribir = permitir;
	}

	/*	Funcion habilitarBarrido(habilitar)
		Habilita y deshabilita el barrido automatico.
	*/
	public function habilitarBarrido(habilitar:Boolean)
	{
		_habilitarBarrido = habilitar;
	}

	//_cambiarFoco()
	private function _cambiarFoco(){
		//orden objetos es un array que contiene undefineds y todos los objetos ordenados por tabindex
		if(_posicionFoco == undefined){
			Selection.setFocus(undefined);
			return;
		}
		if (_ordenObjetos[_posicionFoco] != undefined){
			Selection.setFocus(_ordenObjetos[_posicionFoco]);
			if(_audioDescribir and _permitirAudioDescribir){
				_gAcces.gSincro.audioDescribir(_ordenObjetos[_posicionFoco]);
			}
			_activado = true;
			if ((_tiempoBarrido != 0) or (_pulsadores == true)){
				gEvAct.Notificar(_ordenObjetos[_posicionFoco],"cambio de foco");
			}
		}
		else{
			Selection.setFocus(undefined);
			if(_debug) trace ("ATENCIoN: NO SE ENCONTRo OBJETO AL QUE CAMBIAR EL FOCO");
		}
	}

	//cubrirRaton(yo)
	private function cubrirRaton(yo):Object
	{
		if(_debug) trace("Cubriendo raton "+gEvAct);
		if(_debug) trace("DIMENSIONES:  " + gEvAct._width + "," + gEvAct._height);
		//Creo una caja que dejare oculta para que se detecte el raton tambien en las zonas en las que no
		//hay nada pintado...
		with(gEvAct) {
			createEmptyMovieClip("mc_caja", getNextHighestDepth());
			mc_caja.lineStyle(4, 0x0000FF, 0);
			mc_caja.beginFill(0x0000FF, 0);
			mc_caja.moveTo(0,0);
			mc_caja.lineTo(_width,0);
			mc_caja.lineTo(_width,_height);
			mc_caja.lineTo(0,_height);
			mc_caja.lineTo(0,0);
			mc_caja.endFill();
		}
		if(_debug) trace("gEvAct[mc_caja]" +gEvAct["mc_caja"]);
		gEvAct["mc_caja"].alPulsar = function(){
			if(yo._activado){
				if(_debug) trace(yo + " sobre " + yo._ordenObjetos[yo._posicionFoco]);
				yo.gEvAct.Notificar(yo._ordenObjetos[yo._posicionFoco], "seleccion");
				if(yo.detenerse){
					yo.detener();
				}
			}
			else{
				yo.gEvAct.notificar_a.Notificar(Selection.getFocus(), "seleccion");
			}
		};
		_alPulsarBackUp = gEvAct["mc_caja"].alPulsar;
		return(gEvAct["mc_caja"]);
	}

	/*	descubrirRaton()
		Post: Borra la caja creada por cubrirRaton.
	*/
	public function descubrirRaton()
	{
		gEvAct["mc_caja"].removeMovieClip();
	}

	/*	recuperarBackUp()
		Post: Recupera la funcion alPulsar guardada
	*/
	public function recuperarBackUp()
	{
		gEvAct["mc_caja"].alPulsar = _alPulsarBackUp;
	}

	/*	detener()
		Post: Quita la marca de barrido y detiene temporalmente la temporizacion de barrido.
	*/
	public function detener()
	{
		clearInterval(_intervalo);
		//Le doy un valor 0 al intervalo, a modo de cerradura
		_intervalo = 0;
	}

	/*	resumir()
		Post: Reanuda el barrido automatico.
	*/
	public function resumir()
	{
		if ((_tiempoBarrido != 0) and (_intervalo == 0) and _habilitarBarrido){
			_intervalo = setInterval(this, "avanzar", _tiempoBarrido);
		}
	}

	/*	registrar(obj, indice)
		Pre: Recibe un objeto y una posicion en la que se quiere registrar.
		Post: Añade el objeto al barrido en la posicion tabIndex. Si ya esta usada esta posicion, no se
		introduce el objeto y se traza el error.
	*/
	public function registrar(obj:Object, indice:Number){
		if(_debug) trace("    Registrando barridos de " + obj + " (" + indice + ")");
		if (_ordenObjetos[indice] == undefined)
		{
			_ordenObjetos[indice] = obj;
			//Para cada objeto registrado añado un comportamiento para cuando suelta el oco y cuando lo recoge.
			obj.gBarridos = this;
			obj.onKillFocus = function(){
				gEvAct["marcador"].clear();
			};
			obj.onSetFocus = function(){
				if(_debug) trace("gEvAct: " + this.gBarridos.gEvAct);
				_level0.crearMarcador.call(this.gBarridos.gEvAct,this);
				this.gBarridos._posicionFoco = this.tabIndex;
			}
		}
		else{
			if(_debug) trace("El indice " + indice + " ya esta ocupado: " + _ordenObjetos[indice]);
		}
	}

	/*	liberarObjeto(obj)
		Post: Quita el objeto del barrido.
	*/
	function liberarObjeto(obj:Object)
	{
		if(_debug) trace("Buscando elemento...");
		//Buscare el objeto en todas la lista y lo eliminare
		var i:Number;
		//Primero lo busco en listaCombos3
		for(i=0;i<_ordenObjetos.length;i++)
		{
			if(_ordenObjetos[i] == obj)
			{
				if(_debug) trace("Anulado " + _ordenObjetos[i]);
				//Elimino el elemento iesimo sin mover al resto
				_ordenObjetos[i] = undefined;
				return;
			}
		}
	}

	/*	avanzar()
		Post: Pone el foco en el siguiente objeto registrado.
	*/
	public function avanzar()
	{
		var resultado : Boolean = false;
		if(_debug) trace("AVANZANDO...");
		var grupo:String = _gGrupos.darGrupoBloqueado();
		if(grupo == undefined){
			return;
		}
		detener();
		if (_posicionFoco == _ordenObjetos.length-1){
			_posicionFoco = 0;
		}
		else{
			_posicionFoco++;
		}
		while (!resultado){
			if (_ordenObjetos[_posicionFoco] == undefined){
				if (_posicionFoco==_ordenObjetos.length-1){
					_posicionFoco = 0;
				}
				else{
					_posicionFoco++;
				}
			}
			else {
				if(_ordenObjetos[_posicionFoco].grupo == _gGrupos._grupoBloqueado){
					resultado = true;
				}
				else{
					if(_gAcces.gConfig.GetModoNavegacion() != "NORMAL"){
						_gGrupos.bloqueoGrupo = false;
						for(var i = 0; i < _gGrupos._maestro._listaGrupos.length; i++){
							if(_ordenObjetos[_posicionFoco].grupo == _gGrupos._maestro._listaGrupos[i].darNombre())
							{
								if(_gGrupos._maestro._listaGrupos[i].darDesactivo())
								{
									if (_posicionFoco==_ordenObjetos.length-1){
										_posicionFoco = 0;
									}
									else{
										_posicionFoco++;
									}
								}
								else
								{
									resultado = true;
									break;
								}
							}
						}
						if(_ordenObjetos[_posicionFoco].bloqueado == true)
						{
							if (_posicionFoco==_ordenObjetos.length-1){
								_posicionFoco = 0;
							}
							else{
								_posicionFoco++;
							}
						}
					}
					else{
						if (_posicionFoco==_ordenObjetos.length-1){
							_posicionFoco = 0;
						}
						else{
							_posicionFoco++;
						}
					}
				}
			}
		}
		_cambiarFoco();
		gEvAct.Notificar(_ordenObjetos[_posicionFoco],"avance");
		resumir();
	}

	/*	retroceder()
		Post: Pone el foco en el anterior objeto registrado.
	*/
	public function retroceder()
	{
		var resultado : Boolean = false;
		if(_debug) trace("RETROCEDIENDO...");
		var grupo:String = _gGrupos.darGrupoBloqueado();
		if(grupo == undefined){
			return;
		}
		detener();
		//Busco el primer foco disponible por atras
		if (_posicionFoco == 0){
			_posicionFoco = _ordenObjetos.length-1;
		}
		else{
			_posicionFoco--;
		}	
		while (!resultado){
			if (_ordenObjetos[_posicionFoco] == undefined){
				if (_posicionFoco==0){
					_posicionFoco = _ordenObjetos.length-1;
				}
				else{
					_posicionFoco--;
				}
			}
			else {
				if(_ordenObjetos[_posicionFoco].grupo == _gGrupos._grupoBloqueado){
					resultado =  true;
					}
				else{
					if(_gAcces.gConfig.GetModoNavegacion() != "NORMAL"){
						_gGrupos.bloqueoGrupo = false;
						for(var i = 0; i < _gGrupos._maestro._listaGrupos.length; i++){
							if(_ordenObjetos[_posicionFoco].grupo == _gGrupos._maestro._listaGrupos[i].darNombre())
							{
								if(_gGrupos._maestro._listaGrupos[i].darDesactivo())
								{
									if (_posicionFoco==_ordenObjetos.length-1){
										_posicionFoco = 0;
									}
									else{
										_posicionFoco--;
									}
								}
								else
								{
									resultado = true;
									break;
								}
							}
						}
						if(_ordenObjetos[_posicionFoco].bloqueado == true)
						{
							if (_posicionFoco==_ordenObjetos.length-1){
								_posicionFoco = 0;
							}
							else{
								_posicionFoco--;
							}
						}
					}
					else{
						if (_posicionFoco==0){
							_posicionFoco = _ordenObjetos.length-1;
						}
						else{
							_posicionFoco--;
						}
					}
				}
			}
		}
		_cambiarFoco();
		gEvAct.Notificar(_ordenObjetos[_posicionFoco],"retroceso");
		resumir();
	}

	/*	irA(posicion)
		Post: Pone el foco en el objeto cuyo indice coincide con el pasado como parametro.
	*/
	public function irA(posicion:Number)
	{
		if(posicion == undefined){
			_posicionFoco = undefined;
			_cambiarFoco();
		}
		if (_activado){
			detener();
			_posicionFoco = posicion;
			_cambiarFoco();
			resumir();
		}
		else{
			_posicionFoco = posicion;
			_cambiarFoco();
		}
	}

	/*	bloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Ignora los objetos que se indiquen.
	*/
	public function bloquear(lista:Object, pI:Number, pF:Number)
	{
		if(_debug) trace("Bloqueando Barrido");
		var i:Number;
		var j:Number;
		if (lista != undefined)
		{
			if(pI == undefined){
				pI = 0;
			}
			if(pF == undefined){
				pF = lista.length;
			}
			for(i=pI;i<pF;i++)
			{
				for(j=0;j<_ordenObjetos.length;j++)
				{
					if (_ordenObjetos[j] == lista[i]){
						_ordenObjetos[j].bloqueado = true;
					}
				}
			}
		}
		else
		{
			if(_debug) trace("Bloqueando Barrido");
			//Ahora anulo todos los onPress
			for(i=0;i<_ordenObjetos.length;i++){
				_ordenObjetos[i].bloqueado = true;
			}
		}
	}

	/*	desbloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Recupera el barrido sobre los objetos que se indiquen.
	*/
	public function desbloquear(lista:Object, pI:Number, pF:Number){
		var i:Number;
		var j:Number;
		if (lista != undefined)
		{
			if(pI == undefined){
				pI = 0;
			}
			if(pF == undefined){
				pF = lista.length;
			}
			for (i=pI;i<pF;i++)
			{
				for (j=0;j<_ordenObjetos.length;j++)
				{
					if (_ordenObjetos[j] == lista[i]){
						_ordenObjetos[j].bloqueado = false;
					}
				}
			}
		}
		else
		{
				if(_debug) trace("Desbloqueando Barrido");
				//Ahora restauro todos los onPress
				var i:Number;
				for(i=0;i<_ordenObjetos.length;i++){
					_ordenObjetos[i].bloqueado = false;
				}
		}
	}

	/*	limpiar()
		Post: Deja el gestor preparado para volver a ser utilizado.
	*/
	public function limpiar()
	{
		_activado = false;
		gEvAct["marcador"].clear();
		detener();
		capturadorPulsador.detener();
	}

	/*	Notificar(origen, motivo, info)
		Con esta funcion trato los eventos capturados de raton. No se debe confundir este Notificar con el que
		deberemos implementar dentro de nuestra actividad.
	*/
	function Notificar(origen:MovieClip, motivo: String, info:Object)
	{
		switch(motivo)
		{
			case "pulsado":
				if(_debug) trace("RECOGIDO: " + origen);
				switch(origen)
				{
					case "central":
						if(_debug) trace("HAS PULSADO EL CENTRAL!!!");
						if(gEvAct.gAcces.gConfig.GetModoNavegacion() == "B_MANU"){
							avanzar();
						}
						break;
					case "derecho":
						if(_debug) trace("HAS PULSADO EL DERECHO!!!");
						break;
					case "izquierdo":
						if(_debug) trace("HAS PULSADO EL IZQUIERDO!!!");
						if((_tiempoBarrido != 0 and _activado == true and
							gEvAct.gAcces.gConfig.GetModoNavegacion() == "B_AUTO") or
							(_pulsadores == true) and gEvAct.gAcces.gConfig.GetModoNavegacion() == "B_AUTO")
						{
							gEvAct.Notificar(_ordenObjetos[_posicionFoco],"seleccion");
						}
						break;
					default:
						if(_debug) trace("ERROR EN CAPTURA DE RAToN");
						break;
				}
				break;
			default: 
				if(_debug) trace("Evento no reconocido: "+ motivo +" ("+origen+") {" + info + "}");
				break;
		}
	}
}