﻿/*	Clase configuracion.GestorConfiguracion
	Autor: Alvaro Faraco Martinez
	Esta clase realiza el mantenimiento de la configuracion de la actividad.
*/

import configuracion.GestorSCORM;
import configuracion.GestorAccesibilidad;
import GestorInfoExterna;

class configuracion.GestorConfiguracion
{//Clase configuracion.GestorConfiguracion

	private var _debug:Boolean = false;

	//Atributos privados de la clase
	private var _dificultad:Number;
	private var _modoNavegacion:String;
	private var _intervaloBarrido:Number;
	private var _idioma:String;
	private var _duracionTextos:Number;
	private var _locuciones:Boolean;
	private var _audioDescripcion:Boolean;
	private var _volumen:Number;
	private var _subtitulos:Boolean;
	private var _teclaSeleccion:String;
	private var _teclaAvanceBarrido:String;
	private var _teclaRetrocesoBarrido:String;
	private var _teclaAvanceGrupo:String;
	private var _teclaRetrocesoGrupo:String;
	//El _gestorInfoExterna se utilizara para la comunicacion con el LMS
	private var _gestorInfoExterna:GestorInfoExterna;
	//El _gestorAccesibilidad es el llamante
	private var _gestorAccesibilidad:GestorAccesibilidad;

	/*	GestorConfiguracion(gestorAccesibilidad)
		Pre: recibe el gestor de accesibilidad.
		Post: inicia la comunicacion con el LMS, mediante el gestor SCORM, e inicializa el atributo _gestorAccesibilidad
	*/
	public function GestorConfiguracion(gestorAccesibilidad:GestorAccesibilidad, gestoInfoExterna:GestorInfoExterna)
	{
		_gestorAccesibilidad = gestorAccesibilidad;
		_gestorInfoExterna = gestoInfoExterna;
	}

	/*	GeneralAConfiguracion(general)
		Pre: general es un Object que contiene la configuracion general, extraida del XML de configuracion.
		Post: establece el valor de todos los atributos de configuracion.
	*/
	private function GeneralAConfiguracion(general:Object)
	{
		SetDificultad(Number(general["dificultad"]));
		SetModoNavegacion(String(general["modoNavegacion"]));
		SetIntervaloBarrido(Number(general["intervaloBarrido"]));
		SetIdioma(String(general["idioma"]));
		SetLocuciones(Boolean(Number(general["locuciones"])));
		SetAudioDescripcion(Boolean(Number(general["audioDescripcion"])));
		SetVolumen(Number(general["volumen"]));
		SetSubtitulos(Boolean(Number(general["subtitulos"])));
		SetDuracionTextos(Number(general["duracionTextos"]));
		SetTeclaSeleccion(String(general["teclaSeleccion"]));
		SetTeclaAvanceBarrido(String(general["teclaAvanceBarrido"]));
		SetTeclaRetrocesoBarrido(String(general["teclaRetrocesoBarrido"]));
		SetTeclaAvanceGrupo(String(general["teclaAvanceGrupo"]));
		SetTeclaRetrocesoGrupo(String(general["teclaRetrocesoGrupo"]));
	}

	//ConfiguracionAString(general)
	//Convierte la configuracion en una cadena que se podra guardar en el LMS, en cmi.suspend_data.
	public function ConfiguracionAString():String
	{
		var cadena:String = new String();
		cadena += "<dificultad>" + _dificultad + "</dificultad>";
		cadena += "<modoNavegacion>" + _modoNavegacion + "</modoNavegacion>";
		cadena += "<intervaloBarrido>" + _intervaloBarrido + "</intervaloBarrido>";
		cadena += "<idioma>" + _idioma + "</idioma>";
		cadena += "<duracionTextos>" + _duracionTextos + "</duracionTextos>";
		cadena += "<locuciones>" + Number(_locuciones) + "</locuciones>";
		cadena += "<audioDescripcion>" + Number(_audioDescripcion) + "</audioDescripcion>";
		cadena += "<volumen>" + _volumen + "</volumen>";
		cadena += "<subtitulos>" + Number(_subtitulos) + "</subtitulos>";
		cadena += "<teclaSeleccion>" + _teclaSeleccion + "</teclaSeleccion>";
		cadena += "<teclaAvanceBarrido>" + _teclaAvanceBarrido + "</teclaAvanceBarrido>";
		cadena += "<teclaRetrocesoBarrido>" + _teclaRetrocesoBarrido + "</teclaRetrocesoBarrido>";
		cadena += "<teclaAvanceGrupo>" + _teclaAvanceGrupo + "</teclaAvanceGrupo>";
		cadena += "<teclaRetrocesoGrupo>" + _teclaRetrocesoGrupo + "</teclaRetrocesoGrupo>";
		return cadena;
	}

	//GetParametroString(cadena, parametro)
	//Extrae de la cadena de configuracion el parametro pedido.
	private function GetParametroString(cadena:String, parametro:String):String
	{
		var i:Number = cadena.indexOf("<" + parametro + ">") + parametro.length + 1;
		var j:Number = cadena.indexOf("</" +parametro + ">");
		//Se ha encontrado el inicio y el fin del parametro
		if(i != parametro.length and j != -1)
		{
			//Devuelve el valor del parametro encontrado
			return cadena.substring(++i, j);
		}
		//Devuelve una cadena vacia si no encuentra el parametro
		return "";
	}

	//StringAConfiguracion(cadena)
	//Establece la configuracion extrayendo la informacion de la cadena de configuracion.
	private function StringAConfiguracion(cadena:String)
	{
		SetDificultad(Number(GetParametroString(cadena, "dificultad")));
		SetModoNavegacion(GetParametroString(cadena, "modoNavegacion"));
		SetIntervaloBarrido(Number(GetParametroString(cadena, "intervaloBarrido")));
		SetIdioma(GetParametroString(cadena, "idioma"));
		SetLocuciones(Boolean(Number(GetParametroString(cadena, "locuciones"))));
		SetAudioDescripcion(Boolean(Number(GetParametroString(cadena, "audioDescripcion"))));
		SetVolumen(Number(GetParametroString(cadena, "volumen")));
		SetSubtitulos(Boolean(Number(GetParametroString(cadena, "subtitulos"))));
		SetDuracionTextos(Number(GetParametroString(cadena, "duracionTextos")));
		SetTeclaSeleccion(GetParametroString(cadena, "teclaSeleccion"));
		SetTeclaAvanceBarrido(GetParametroString(cadena, "teclaAvanceBarrido"));
		SetTeclaRetrocesoBarrido(GetParametroString(cadena, "teclaRetrocesoBarrido"));
		SetTeclaAvanceGrupo(GetParametroString(cadena, "teclaAvanceGrupo"));
		SetTeclaRetrocesoGrupo(GetParametroString(cadena, "teclaRetrocesoGrupo"));
	}

	/*	Conjunto de funciones que guardan, comprueban y recuperan el string de configuracion.
		(1) GuardarConfiguracion() guarda el string de configuracion.
		(2) HayConfiguracionGuardada() comprueba si hay un string de configuracion guardado.
		(3) RecuperarConfiguracionGuardada() aplica la configuracion guardada en el string de configuracion.
	*/
	//GuardarConfiguracion
	public function GuardarConfiguracion()
	{
		_level0.stringConfiguracion = ConfiguracionAString();
		_gestorInfoExterna.guardaInfoExterna();
	}
	//HayConfiguracionGuardada
	public function HayConfiguracionGuardada()
	{
		return (_level0.stringConfiguracion != undefined);
	}
	//RecuperarConfiguracionGuardada
	public function RecuperarConfiguracionGuardada()
	{
		StringAConfiguracion(_level0.stringConfiguracion);
	}


	/*	SetConfiguracion(general)
		Pre: general es un Object que contiene la configuracion general, extraida del XML de configuracion.
		Post: establece la configuracion de la actividad, ya sea extrayendola de cmi.suspend_data, cuando hay
		conexion con el LMS, o del XML de configuracion. Si cmi.suspend_data no tiene contenido, escribe en el
		la cadena de configuracion.
	*/
	public function SetConfiguracion(general:Object)
	{
		if(_debug) trace("*** Estableciendo la configuracion general...");
		
		var configuracion:String = _gestorInfoExterna.leeInfoExterna();

		if ( !configuracion )
		{
			GeneralAConfiguracion(general);
		}
		else
		{
			StringAConfiguracion(configuracion);
		}
		GuardarConfiguracion();
	}

	/*	GetDificultad()
		Post: devuelve el valor del atributo _dificultad.
	*/
	public function GetDificultad():Number
	{
		return _dificultad;
	}

	/*	SetDificultad(dificultad)
		Post: establece el valor del atributo _dificultad.
	*/
	public function SetDificultad(dificultad:Number)
	{
		_dificultad = dificultad;
		_level0.dificultad = dificultad;
	}


	/*	GetModoNavegacion()
		Post: devuelve el valor del atributo _modoNavegacion.
	*/
	public function GetModoNavegacion():String
	{
		return _modoNavegacion;
	}

	/*	SetModoNavegacion(modoNavegacion)
		Pre: modoNavegacion debe ser "NORMAL", "B_AUTO" o "B_MANU".
		Post: establece el valor del atributo _modoNavegacion y aplica el cambio.
	*/
	public function SetModoNavegacion(modoNavegacion:String)
	{
		_modoNavegacion = modoNavegacion;
		switch(_modoNavegacion)
		{
			case "NORMAL":
				//Control por teclado y raton
				_gestorAccesibilidad.barridoAutomatico(0);
				_gestorAccesibilidad.gBarridos.capturadorPulsador.Liberar();
				_gestorAccesibilidad.gBarridos.descubrirRaton();
				_gestorAccesibilidad.gBarridos.recuperarBackUp();
				break;
			case "B_AUTO":
				//Barrido automatico
				_gestorAccesibilidad.barridoAutomatico(_intervaloBarrido);
				break;
			case "B_MANU":
				//Barrido por pulsadores
				_gestorAccesibilidad.barridoAutomatico(-1);
				break;
		}
	}

	/*	GetIntervaloBarrido()
		Post: devuelve el valor del atributo _intervaloBarrido.
	*/
	public function GetIntervaloBarrido():Number
	{
		return _intervaloBarrido;
	}

	/*	SetIntervaloBarrido(intervaloBarrido)
		Post: establece el valor del atributo _intervaloBarrido.
	*/
	public function SetIntervaloBarrido(intervaloBarrido:Number)
	{
		_intervaloBarrido = intervaloBarrido;
	}

	/*	GetIdioma()
		Post: devuelve el valor del atributo _idioma.
	*/
	public function GetIdioma():String
	{
		return _idioma;
	}

	/*	SetIdioma(idioma)
		Post: establece el valor del atributo _idioma.
	*/
	public function SetIdioma(idioma:String)
	{
		_idioma = idioma;
	}

	/*	GetDuracionTextos()
		Post: devuelve el valor del atributo _duracionTextos.
	*/
	public function GetDuracionTextos():Number
	{
		return _duracionTextos;
	}

	/*	SetDuracionTextos(duracionTextos)
		Post: establece el valor del atributo _duracionTextos y establece la duracion de los textos.
	*/
	public function SetDuracionTextos(duracionTextos:Number)
	{
		_duracionTextos = duracionTextos;
		_gestorAccesibilidad.gSincro.gSubtitulos.setDelay(_duracionTextos);
	}

	/*	GetLocuciones()
		Post: devuelve el valor del atributo _locuciones.
	*/
	public function GetLocuciones():Boolean
	{
		return _locuciones;
	}

	/*	SetLocuciones(locuciones)
		Post: establece el valor del atributo _locuciones y activa o desactiva las locuciones.
	*/
	public function SetLocuciones(locuciones:Boolean)
	{
		_locuciones = locuciones;
		if(_locuciones)
		{
			_gestorAccesibilidad.gSincro.gSubtitulos.activarAudio();
		}
		else
		{
			_gestorAccesibilidad.gSincro.gSubtitulos.desactivarAudio();
		}
	}

	/*	GetAudioDescripcion()
		Post: devuelve el valor del atributo _audioDescripcion.
	*/
	public function GetAudioDescripcion():Boolean
	{
		return _audioDescripcion;
	}

	/*	SetAudioDescripcion(audioDescripcion)
		Post: establece el valor del atributo _audioDescripcion y aplica el cambio.
	*/
	public function SetAudioDescripcion(audioDescripcion:Boolean)
	{
		_audioDescripcion = audioDescripcion;
		_gestorAccesibilidad.gBarridos.audioDescribir(audioDescripcion);
	}

	/*	GetVolumen()
		Post: devuelve el valor del atributo _volumen.
	*/
	public function GetVolumen():Number
	{
		return _volumen;
	}

	/*	SetVolumen(volumen)
		Post: establece el valor del atributo _volumen y establece el volumen del audio.
	*/
	public function SetVolumen(volumen:Number)
	{
		//Se controla que el volumen este entre 0 y 100
		if(volumen > 100){
			_volumen = 100;
		}
		else if(volumen < 0){
			_volumen = 0;
		}
		else{
			_volumen = volumen;
		}
		_gestorAccesibilidad.gSincro.gSubtitulos.setVolumen(_volumen);
		//Movemos el barra de control de volumen al aplicar el cambio de volumen
		_level100.sonidoInit(_volumen);
		_level100.mueveDeslizador(_volumen);
	}

	/*	GetSubtitulos()
		Post: devuelve el valor del atributo _subtitulos.
	*/
	public function GetSubtitulos():Boolean
	{
		return _subtitulos;
	}

	/*	SetSubtitulos(subtitulos)
		Post: establece el valor del atributo _subtitulos y activa o desactiva los subtitulos.
	*/
	public function SetSubtitulos(subtitulos:Boolean)
	{
		_subtitulos = subtitulos;
		if(_subtitulos)
		{
			_gestorAccesibilidad.gSincro.gSubtitulos.activarSubs();
		}
		else
		{
			_gestorAccesibilidad.gSincro.gSubtitulos.desactivarSubs();
		}
	}

	/*	Conjunto de funciones que guardan o recogen el valor de los atributos de configuracion de teclado.
		(1) Tecla de seleccion.
		(2) Tecla de avance de barrido dentro de un mismo grupo.
		(3) Tecla de retroceso de barrido dentro de un mismo grupo.
		(4) Tecla de avance de grupo de seleccion.
		(5) Tecla de restroceso de grupo de seleccion.
	*/
	//GetTeclaSeleccion()
	public function GetTeclaSeleccion():String
	{
		return _teclaSeleccion;
	}
	//SetTeclaSeleccion(teclaSeleccion)
	public function SetTeclaSeleccion(teclaSeleccion:String)
	{
		_teclaSeleccion = teclaSeleccion;
	}
	//GetTeclaAvanceBarrido()
	public function GetTeclaAvanceBarrido():String
	{
		return _teclaAvanceBarrido;
	}
	//SetTeclaAvanceBarrido(teclaAvanceBarrido)
	public function SetTeclaAvanceBarrido(teclaAvanceBarrido:String)
	{
		_teclaAvanceBarrido = teclaAvanceBarrido;
	}
	//GetTeclaRetrocesoBarrido()
	public function GetTeclaRetrocesoBarrido():String
	{
		return _teclaRetrocesoBarrido;
	}
	//SetTeclaRetrocesoBarrido(teclaRetrocesoBarrido)
	public function SetTeclaRetrocesoBarrido(teclaRetrocesoBarrido:String)
	{
		_teclaRetrocesoBarrido = teclaRetrocesoBarrido;
	}
	//GetTeclaAvanceGrupo()
	public function GetTeclaAvanceGrupo():String
	{
		return _teclaAvanceGrupo;
	}
	//SetTeclaAvanceGrupo(teclaAvanceGrupo)
	public function SetTeclaAvanceGrupo(teclaAvanceGrupo:String)
	{
		_teclaAvanceGrupo = teclaAvanceGrupo;
	}
	//GetTeclaRetrocesoGrupo()
	public function GetTeclaRetrocesoGrupo():String
	{
		return _teclaRetrocesoGrupo;
	}
	//SetTeclaRetrocesoGrupo(teclaRetrocesoGrupo)
	public function SetTeclaRetrocesoGrupo(teclaRetrocesoGrupo:String)
	{
		_teclaRetrocesoGrupo = teclaRetrocesoGrupo;
	}

}//Clase GestorConfiguracion
