﻿/*	Clase configuracion.GestorRaton
	Esta clase encapsula los sucesos de pulsacion de raton para generar notificaciones.
	Se introduce cada objeto registrado en una lista y ademas se sustituye su metodo onPress
	por un metodo propio de notificacion. Se guarda un backup de dicho metodo para restaurarlo
	al liberarlo.
*/

import configuracion.Notificacion

class configuracion.GestorRaton extends Object
{
	private var _debug:Boolean = false;

	//Atributos privados de la clase
	private var notificar_a:Object;
	//Lista de asociaciones tecla-simbolo
	private var listaEltosEscena:Array;
	private var pos_x:Number;
	private var pos_y:Number;
	private var bloqueado:Boolean;

	/*	GestorRaton(notificar_a)
		Constructor de la clase
		Pre: Recibe el gestor de eventos que recibira las notificaciones capturadas.
		Post: Inicializa los atributos necesarios y establece la clase como escuchador del raton.
	*/
	function GestorRaton(notificar_a:Object)
	{
		this.notificar_a = notificar_a;
		this.listaEltosEscena = new Array();
		this.bloqueado = false;
		//Asignamos al GestorRaton como escuchador de eventos
		Mouse.addListener(this);
	}

	/*	LiberarRaton()
		Post: Libera el escuchador de raton y destruye la instancia, liberando todos los objetos
		registrados. Recupera los metodos onPress originales.
	*/
	function LiberarRaton()
	{
		Mouse.removeListener(this);
		//Ahora recupero todos los onPress originales
		var i:Number;
		for(i=0; i<listaEltosEscena.length; i++)
		{
			listaEltosEscena[i].onPress = listaEltosEscena[i].onPressBackup;
			delete listaEltosEscena[i].onPressBackup;
		}
		delete this;
	}

	/*	Registrar(obj)
		Post: Añade el objeto que recibe como parametro al tratamiento.
	*/
	function Registrar(obj:Object)
	{
		listaEltosEscena.push(obj);
		obj.notificar_a = this.notificar_a;
		if (obj.onPressBackup == null){
			if(_debug) trace("    " + obj +".onPress() ya existia, guardando copia...");
			obj.onPressBackup = obj.onPress; //Guardo el onPress original
			obj.onPress= function()
			{
				if(_debug) trace("Raton esta sobre "+ this);
				notificar_a.Notificar(this,"seleccion",true);
			}
		}
		else
			if(_debug) trace("Ya habia onPressBackup!!!");
	}

	/*	Liberar(obj)
		Post: Quita el objeto que recibe como parametro del tratamiento.
	*/
	function Liberar(obj:Object)
	{
		var i:Number=0;
		while(i<listaEltosEscena.length)
		{
			if(_debug) trace("comparando: "+obj+" "+listaEltosEscena[i]);
			if(obj == listaEltosEscena[i])
			{
				//Elimino el elemento iesimo
				listaEltosEscena.splice(i, 1);
				delete obj.notificar_a;
				//Restauro el onPress original
				obj.onPress = obj.onPressBackup;
				delete obj.onPressBackup;
				if(_debug) trace("Anulado " + obj);
				return;
			}
			i++;
		}
		//Si llega hasta aqui significa que no ha encontrado el objeto
		if(_debug) trace(obj+" no encontrado!!!");
		return -1;
	}

	/*	Bloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Bloquea temporalmente la captura de eventos de raton.
	*/
	public function Bloquear(lista:Object, pI:Number, pF:Number)
	{
		var i:Number;
		var j:Number;
		if(lista != undefined)
		{
			if(pI == undefined){
				pI = 0;
			}
			if(pF == undefined){
				pF = lista.length;
			}
			for(i=pI; i<pF; i++)
			{
				for (j=0; j<listaEltosEscena.length; j++)
				{
					if (listaEltosEscena[j] == lista[i]){
						listaEltosEscena[j].onPress = function(){};
					}
				}
			}
		}
		else
		{
			if(not bloqueado)
			{
				if(_debug) trace("Bloqueando Raton");
				//Ahora anulo todos los onPress
				for(i=0; i<listaEltosEscena.length; i++)
				{
					listaEltosEscena[i].onPress = function(){};
				}
			}
			bloqueado = true;
		}
	}

	/*	Desbloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Restaura la captura de eventos de raton.
	*/
	public function Desbloquear(lista:Object, pI:Number, pF:Number)
	{
		var i:Number;
		var j:Number;
		if(lista != undefined)
		{
			if(pI == undefined){
				pI = 0;
			}
			if(pF == undefined){
				pF = lista.length;
			}
			for(i=pI; i<pF; i++)
			{
				for (j=0; j<listaEltosEscena.length; j++)
				{
					if (listaEltosEscena[j] == lista[i])
					{
						listaEltosEscena[j].onPress = function(){
							notificar_a.Notificar(this,"seleccion",true);
						}
					}
				}
			}
		}
		else
		{
			if(bloqueado)
			{
				if(_debug) trace("Desloqueando Raton");
				//Ahora restauro todos los onPress
				var i:Number;
				for(i=0; i<listaEltosEscena.length; i++)
				{
					listaEltosEscena[i].onPress = function(){
						notificar_a.Notificar(this,"seleccion",true);
					}
				}
			}
			bloqueado = false;
		}
	}
}
