﻿/*	Clase configuracion.GestorSCORM
	Autor: Alvaro Faraco Martinez
	Fecha: 14 de mayo de 2008
	Esta clase proporciona servicios para facilitar la comunicacion entre un
	documento Flash y un LMS compatible con SCORM 2004. Para su correcto
	funcionamiento es necesario que el HTML en el que esta situado dicho documento
	Flash importe el archivo de JavaScript APIWrapper.js, a traves del cual se
	realiza realmente la comunicacion con el LMS.
*/

import flash.external.ExternalInterface;

class configuracion.GestorSCORM
{//Clase configuracion.GestorSCORM

	//Atributos privados de la clase
	//_commitNeeded indica si es necesario actualizar los datos
	private var _commitNeeded:Boolean;

	/*	GestorSCORM()
		Post: Contructor de la clase, inicia _commitNeeded a falso.
	*/
	public function GestorSCORM()
	{
		_commitNeeded = false;
	}

	/*	IsCommitNeeded()
		Post: Devuelve el valor del atributo _commitNeeded.
	*/
	public function IsCommitNeeded():Boolean
	{
		return _commitNeeded;
	}

	/*	Initialize()
		Post: Inicia la comunicacion con el LMS. Devuelve true si se establece la comunicacion y false en caso
		contrario.
	*/
	public function Initialize():Boolean
	{
		var retorno:Boolean = Boolean(ExternalInterface.call("doInitialize"));
		return retorno;
	}

	/*	IsInitialized()
		Post: Indica si estamos en comunicacion con el LMS. Devuelve true si la comunicacion esta establecida y
		false en caso contrario.
	*/
	public function IsInitialized():Boolean
	{
		var retorno:Boolean = Boolean(ExternalInterface.call("LMSIsInitialized"));
		return retorno;
	}

	/*	Terminate()
		Post: Termina la comunicacion con el LMS. Devuelve true si termina la comunicacion y false en caso
		contrario.
	*/
	public function Terminate():Boolean
	{
		var retorno:Boolean = Boolean(ExternalInterface.call("doTerminate"));
		if (retorno == true)
		{
			//Si se ha terminado correctamente, se ha hecho actualizacion
			_commitNeeded = false;
		}
		return retorno;
	}

	/*	GetValue(elemento)
		Pre: Recibe un string que identifica un elemento del Scorm 2004 RTE Data Model(*).
		Post: Devuelve el valor de un elemento del modelo de datos.
		(*) En el documento SCORM 2004 hay una lista de los elementos que tiene el modelo.
	*/
	public function GetValue(elemento:String):String
	{
		var retorno:String = String(ExternalInterface.call("doGetValue", elemento));
		return retorno;
	}

	/*	SetValue(elemento,valor)
		Pre: Recibe un string que identifica un elemento del Scorm 2004 RTE Data Model(*).
		Post: Establece el valor de un elemento del modelo de datos.
		(*) En el documento SCORM 2004 hay una lista de los elementos que tiene el modelo.
	*/
	public function SetValue(elemento:String,valor:String):Boolean
	{
		var retorno:Boolean = Boolean(ExternalInterface.call("doSetValue", elemento, valor));
		if (retorno == true)
		{
			//Es necesario actualizar
			_commitNeeded = true;
		}
		return retorno;
	}

	/*	Commit()
		Post: Actualiza los datos del LMS con los datos guardados en cache.
	*/
	public function Commit():Boolean
	{
		var retorno:Boolean = Boolean(ExternalInterface.call("doCommit"));
		if (retorno == true)
		{
			//Se ha hecho la actualizacion correctamente
			_commitNeeded = false;
		}
		return retorno;
	}

	/*	GetLastError()
		Post: Devuelve el codigo del ultimo error producido.
	*/
	public function GetLastError():String
	{
		var retorno:String = String(ExternalInterface.call("doGetLastError"));
		return retorno;
	}

	/*	GetErrorString(error)
		Pre: Recibe un string que identifica a un codigo de error.
		Post: Devuelve el string correspondiente al codigo de error.
	*/
	public function GetErrorString(error:String):String
	{
		var retorno:String = String(ExternalInterface.call("doGetErrorString", error));
		return retorno;
	}

	/*	GetDiagnostic(error)
		Pre: Recibe un string que identifica a un codigo de error.
		Post: Devuelve mas detalles sobre el error correspondiente.
	*/
	public function GetDiagnostic(error:String):String
	{
		var retorno:String = String(ExternalInterface.call("doGetDiagnostic", error));
		return retorno;
	}

}//Clase GestorSCO
