﻿/*	Clase configuracion.GestorTeclado
	Lleva el control de asociaciones teclas-objetos y combinaciones.
*/

import configuracion.Notificacion;
import configuracion.datatypes.TCombinacion;
import configuracion.datatypes.MetodosAbreviados;

class configuracion.GestorTeclado extends Object
{
	private var _debug:Boolean = false;

	//Atributos privados de la clase
	private var notificar_a:Object;
	private var abreviaturas:MetodosAbreviados;
	private var listaTeclas:Array; //lista de asociaciones tecla-simbolo
	private var listaCombos:Array; //lista de combinaciones de 2 teclas
	private var bloqueado:Boolean;

	/*	GestorTeclado(notificar_a)
		Constructor de la clase
		Pre: Recibe el gestor de eventos que recibira las notificaciones capturadas.
		Post: Inicializa los atributos necesarios y establece la clase como escuchador del teclado.
	*/
	function GestorTeclado(notificar_a:Object)
	{
		this.notificar_a = notificar_a;
		this.listaTeclas = new Array();
		this.listaCombos = new Array();
		this.bloqueado = false;
		//Asignamos al GestorTeclado como escuchador de eventos
		Key.addListener(this);
		//Creamos el traductor de metodos abreviados
		abreviaturas = new MetodosAbreviados();
	}
	
	/*	LiberarTeclado()
		Post: Libera el escuchador de teclado y destruye la instancia, liberando todos los objetos
		registrados.
	*/
	function LiberarTeclado()
	{
		Key.removeListener(this);
		//Tengo que liberar todas las abreviaturas asociadas a los objetos registrados.
		var i:Number;
		for(i=0; i<listaCombos.length; i++)
		{
			listaCombos[i]._accProps.shortcut = null;
			listaCombos.splice(i, 1);
		}
		for(i=0; i<listaTeclas.length; i++)
		{
			listaTeclas[i]._accProps.shortcut = null;
			listaTeclas[i] = null;
		}
		Accessibility.updateProperties();
		delete this;
	}

	/*	onKeyDown()
		Metodo que recoge los eventos de tecla pulsada.
	*/
	function onKeyDown()
	{
		if(bloqueado){
			return;
		}
		//Primero miro las combinaciones de 3 teclas, porque son mas restrictivas
		var i:Number;
		for(i=0; i<listaCombos.length; i++)
		{
			if ((!listaCombos[i].bloqueado) and
				((listaCombos[i].tecla1 != null) and (Key.isDown(listaCombos[i].tecla1))) and
				((listaCombos[i].tecla2 != null) and (Key.isDown(listaCombos[i].tecla2))) and
				((listaCombos[i].tecla3 == null) or (Key.isDown(listaCombos[i].tecla3))))
			{
				if(_debug) trace("pulsada " + listaCombos[i].tecla1 + " + " +
											  listaCombos[i].tecla2 + " + " +
											  listaCombos[i].tecla3);
				notificar_a.Notificar(listaCombos[i].objeto, "seleccion", false);
				return;
			}
		}
		//Notificamos solo para aquelos que esten registrados
		if(_debug) trace(listaTeclas[Key.getCode().toString()].bloqueado);
		if (((listaTeclas[Key.getCode().toString()].bloqueado == undefined) or
			(!listaTeclas[Key.getCode().toString()].bloqueado)) and
			(listaTeclas[Key.getCode().toString()] != null))
		{
			notificar_a.Notificar(listaTeclas[Key.getCode().toString()],"seleccion",false);
		}
	}

	/*	RegistrarKeyCode(keyCode, simbolo)
		Pre: Recibe una tecla expresada en forma de codigo de tecla (keyCode) y un objeto (simbolo).
		Post: Inserta una nueva asociacion de tecla-objeto en la lista.
	*/
	function RegistrarKeyCode(keyCode:Number, simbolo:Object)
	{
		if ((listaTeclas[keyCode.toString()] == null) or
			(listaTeclas[keyCode.toString()] == undefined))
		{
			listaTeclas[keyCode.toString()]=simbolo;
			if(_debug) trace("    Registrado: " + keyCode + " sobre " + simbolo);
			simbolo._accProps.shortcut = abreviaturas.teclaAString(keyCode);
			Accessibility.updateProperties();
		}
	}

	/*	RegistrarComboKeyCode(tecla1, tecla2, tecla3, simbolo)
		Pre: Recibe hasta 3 teclas expresadas en forma de codigo de tecla y un objeto (simbolo).
		Post: Inserta una nueva asociacion de combinacion de teclas-objeto en la lista.
	*/
	function RegistrarComboKeyCode(tecla1:Number, tecla2:Number, tecla3:Number, simbolo:Object)
	{
		//Genero el objeto y lo inserto en la lista correspondiente
		var strCombi:String = abreviaturas.teclaAString(tecla1) + " + " + 
							  abreviaturas.teclaAString(tecla2) + " + " + 
							  abreviaturas.teclaAString(tecla3);
		if(_debug) trace(strCombi);
		if(not Disponible(strCombi))
		{
			//Devuelvo -1 si no esta disponible la combinacion
			return -1;
		}
		var obj:TCombinacion = new TCombinacion(tecla1, tecla2, tecla3, strCombi);
		obj.objeto = simbolo;
		if (obj.tecla2 != undefined)
		{
			listaCombos.push(obj);
			if(_debug) trace("    Registrado: " + obj.tecla1 + " " +
												  obj.tecla2 + " " +
												  obj.tecla3 + " sobre " + obj.objeto);
		}
		else
		{
			listaTeclas[tecla1.toString()]=simbolo;
			if(_debug) trace("    Registrado: " + tecla1 + " sobre " + obj.objeto);
		}
		simbolo._accProps.shortcut = strCombi;
		Accessibility.updateProperties();
	}

	/*	Registrar(combinacion, simbolo)
		Pre: Recibe una combinacion expresada en forma de cadena de texto y un objeto (simbolo).
		Post: Inserta una nueva asociacion de combinacion de teclas-objeto en la lista.
	*/
	function Registrar(combinacion:String, simbolo:Object)
	{
		if(_debug) trace("*** Registrando...");
		if (not Disponible(combinacion))
		{
			//Devuelvo -1 si no esta disponible la combinacion
			return -1;
		}
		//Traduzco la combinacion al objeto combinacion y lo inserto en la lista correspondiente
		var obj:TCombinacion;
		obj = abreviaturas.cadenaATeclas(combinacion);
		obj.objeto = simbolo;
		if((obj.tecla2 != undefined) and (obj.tecla3 != undefined))
		{
			listaCombos.push(obj);
			if(_debug) trace("    Registrado: " + obj.tecla1 + " " +
												  obj.tecla2 + " " +
												  obj.tecla3 + " sobre " + obj.objeto);
		}
		else if(obj.tecla2 != undefined)
		{
			listaCombos.push(obj);
			if(_debug) trace("    Registrado: " + obj.tecla1 + " " +
												  obj.tecla2 + " sobre " + obj.objeto);
		}
		else
		{
			listaTeclas[obj.tecla1.toString()]=simbolo;
			if(_debug) trace("    Registrado: "+ obj.tecla1 +" sobre "+obj.objeto);
		}
		//Registro el valor del shortcut en el objeto
		simbolo._accProps.shortcut = combinacion;
		Accessibility.updateProperties();
	}

	/*	LiberarKeyCode(keyCode)
		Pre: Recibe una tecla expresada en forma de codigo de tecla (keyCode).
		Post: Libera una tecla.
	*/
	function LiberarKeyCode(keyCode:Number)
	{
		listaTeclas[keyCode.toString()] = null;
	}

	/*	LiberarComboKeyCode(tecla1, tecla2, tecla3)
		Pre: Recibe hasta 3 teclas expresadas en forma de codigo de tecla.
		Post: Libera una combinacion de teclas.
	*/
	function LiberarComboKeyCode(tecla1:Number, tecla2:Number, tecla3:Number)
	{
		var i:Number=0;
		if(_debug) trace("Llego con: "+tecla1+" "+tecla2+" "+tecla3);
		//Ahora hago la busqueda
		while (i<listaCombos.length){
			if ((listaCombos[i].tecla1 == tecla1) and
				(listaCombos[i].tecla2 == tecla2) and
				((tecla3 == null) or (listaCombos[i].tecla3 == tecla3)))
			{
				if(_debug) trace("Anulado " + listaCombos[i].tecla1 + " " +
											  listaCombos[i].tecla2 + " " +
											  listaCombos[i].tecla3 +
											  " (" + listaCombos[i].objeto + ")");
				//Libero el shortcut en el objeto
				listaCombos[i]._accProps.shortcut = null;
				Accessibility.updateProperties();
				//Elimino el elemento iesimo
				listaCombos.splice(i,1);
				return;
			}
			i++;
		}
		//Si llega hasta aqui significa que no ha encontrado el objeto
		if(_debug) trace(tecla1+" "+tecla2+" "+tecla3+" no encontrado");
		return -1;
	}

	/*	Liberar(combinacion)
		Pre: Recibe una combinacion expresada en forma de cadena de texto.
		Post: Libera una tecla o una combinacion de teclas.
	*/
	function Liberar(combinacion:String)
	{
		var combi:TCombinacion = abreviaturas.cadenaATeclas(combinacion);
		if (combi.tecla2 == undefined){
			return LiberarKeyCode(combi.tecla1);
		}
		else{
			return LiberarComboKeyCode(combi.tecla1, combi.tecla2, combi.tecla3);
		}
	}

	/*	LiberarObjeto(obj)
		Pre: Recibe un objeto.
		Post: Quita un objeto del tratamiento, liberando tambien todas sus asociaciones de teclas y
		combinaciones.
	*/
	function LiberarObjeto(obj:Object)
	{
		//Buscare el objeto en todas las listas y lo eliminare
		var i:Number;
		//Primero lo busco en listaCombos3
		for(i=0;i<listaCombos.length;i++)
		{
			if (listaCombos[i].objeto == obj)
			{
				if(_debug) trace("Anulado " + abreviaturas.teclaAString(listaCombos[i].tecla1) + " " +
											  abreviaturas.teclaAString(listaCombos[i].tecla2) + " " +
											  abreviaturas.teclaAString(listaCombos[i].tecla3) +
											  " (" + obj + ")");
				//Elimino el elemento iesimo
				listaCombos.splice(i,1);
			}
		}
		//Y para terminar, lo quito de listaTeclas
		for(i=0;i<listaTeclas.length;i++)
		{
			if (listaTeclas[i] == obj)
			{
				if(_debug) trace("Anulado "+i);
				listaTeclas[i]=null;
			}
		}
		//Libero el shortcut en el objeto
		obj._accProps.shortcut = null;
		Accessibility.updateProperties();
	}

	/*	Disponible(combinacion)
		Pre: Recibe una combinacion expresada en forma de cadena de texto.
		Post: Busca la combinacion y devuelve cierto si no esta usada.
	*/
	function Disponible(combinacion:String):Boolean
	{
		var combi:TCombinacion = abreviaturas.cadenaATeclas(combinacion);
		if(combi.tecla1 == undefined){
			//Miro si es una combinacion util
			return false;
		}
		else{
			if (combi.tecla2 == undefined)
				//Miro si tiene mas de un elemento.
				return ((listaTeclas[combi.tecla1.toString()] == null) or
						(listaTeclas[combi.tecla1.toString()] == undefined));
			else{
				//Hago la busqueda en listaCombos
				for (var i = 0;i < listaCombos.length;i++){
				  if ((listaCombos[i].tecla1 == combi.tecla1) and
					  (listaCombos[i].tecla2 == combi.tecla2) and
					  (listaCombos[i].tecla3 == combi.tecla3))
					  return false;
				}
			}
		}
		//Si no o ha encontrado, esta disponible.
		return true;
	}

	/*	Bloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Bloquea temporalmente la captura de eventos de teclado.
	*/
	public function	Bloquear(lista:Object, pI:Number, pF:Number)
	{
		if(_debug) trace("Solicitud de bloqueo para " + lista + " desde " + pI + " hasta " + pF);
		var i:Number;
		var j:Number;
		var jMax:Number;
		if(pI == undefined){
			pI = 0;
		}
		if(pF == undefined){
			pF = lista.length;
		}
		if(lista != undefined)
		{
			//Para cada elemento, lo busco en mi lista de combos y lo bloqueo
			for (i=pI;i<pF;i++)
			{
				if (listaCombos.length > listaTeclas.length){
					jMax = listaCombos.length;
				}
				else{
					jMax = listaTeclas.length;
				}
				j=0;
				while (j<jMax)
				{
					if ((j<listaCombos.length) and (listaCombos[j].objeto == lista[i]))
					{
						if(_debug) trace("Bloqueando combo " + listaCombos[j].objeto);
				 		listaCombos[j].bloqueado = new Boolean(true);
						break;
					}
					if ((j<listaTeclas.length) and (listaTeclas[j] == lista[i]))
					{
						listaTeclas[j].bloqueado = new Boolean(true);
						if(_debug) trace("Bloqueando tecla " + listaTeclas[j].bloqueado +
										 "(" + lista[i] + ")");
						break;
					}
					j++;
				}
				if (j == jMax){
					if(_debug) trace("No se encontro");
				}
			}
		}
		else{
			bloqueado = true;
		}
	}

	/*	Desbloquear(lista, pI, pF)
		Pre: Se puede pasar una lista de objetos a los que aplicar esta situacion. Asimismo, se
		puede especificar el rango de actuacion dentro de la lista pasada, mediante los parametros
		inicial y final.
		Post: Restaura la captura de eventos de teclado.
	*/
	public function Desbloquear(lista:Array, pI:Number, pF:Number)
	{
		var i:Number;
		var j:Number;
		var jMax:Number;
		if(pI == undefined){
			pI = 0;
		}
		if(pF == undefined){
			pF = lista.length;
		}
		if(lista != undefined)
		{
			//Para cada elemento, lo busco en mi lista de combos y lo bloqueo
			for (i=pI;i<pF;i++)
			{
				if(listaCombos.length > listaTeclas.length){
					jMax = listaCombos.length;
				}
				else{
					jMax = listaTeclas.length;
				}
				j=0;
				while(j<jMax)
				{
					if((j<listaCombos.length) and (listaCombos[j].objeto == lista[i]))
					{
				 		delete listaCombos[j].bloqueado;
						break;
					}
					if((j<listaTeclas.length) and (listaTeclas[j] == lista[i]))
					{
						delete listaTeclas[j].bloqueado;
						break;
					}
					j++;
				}
				if(j == jMax){
					if(_debug) trace("No se encontro");
				}
			}
		}
		else{
			bloqueado = false;
		}
	}
}
