﻿/*	Clase configuracion.datatypes.Tecla
	Autor: Alvaro Faraco Martinez
	Fecha: 3 de junio de 2008
	Esta clase define una Tecla. Las Teclas solo tienen un atributo: un numero de 
	codigo que representa a cada una de las teclas de un teclado QWERTY estandar. 
	Para crear una instancia de esta clase se pueden utilizar sus atributos 
	estaticos. Por ejemplo:
	var tecla:Tecla() = new Tecla(Tecla.SPACE);
	var tecla:Tecla() = new Tecla(32);
	Las dos instancias anteriores serian equivalentes.
*/

class configuracion.datatypes.Tecla
{//Clase configuracion.datatypes.Tecla

	//Atributos privados de la clase
	private var _codigo:Number;
	private var _combinacion:Array;

	//Atributos estaticos de la clase
	//Solo se definen los numeros de codigo mas comunes
	public static var ALT:Number = 18;
	public static var BACKSPACE:Number = 8;
	public static var CAPS_LOCK:Number = 20;
	public static var CONTROL:Number = 17;
	public static var DELETE:Number = 46;
	public static var DOWN:Number = 40;
	public static var END:Number = 35;
	public static var ENTER:Number = 13;
	public static var ESCAPE:Number = 27;
	public static var HOME:Number = 36;
	public static var INSERT:Number = 45;
	public static var LEFT:Number = 37;
	public static var PAGE_DOWN:Number = 34;
	public static var PAGE_UP:Number = 33;
	public static var RIGHT:Number = 39;
	public static var SHIFT:Number = 16;
	public static var SPACE:Number = 32;
	public static var TAB:Number = 9;
	public static var UP:Number = 38;

	//Constructor de la clase
	public function Tecla(codigo:Number)
	{
		_codigo = codigo;
		_combinacion = new Array();
	}

	//GetCodigo()
	//Devuelve el valor del atributo _codigo
	public function GetCodigo():Number
	{
		return _codigo;
	}

	//SetCodigo()
	//Establece el valor del atributo _codigo
	public function SetCodigo(codigo:Number)
	{
		_codigo = codigo;
	}

	//GetCombinacion()
	//Devuelve el valor del atributo _combinacion
	public function GetCombinacion():Array
	{
		return _combinacion;
	}

	//SetCombinacion()
	//Establece el valor del atributo _combinacion
	public function SetCombinacion(combinacion:Array)
	{
		_combinacion = combinacion;
	}

	//Texto()
	//Devuelve el String correspondiente a una Tecla
	public function Texto():String
	{
		if(_combinacion.length == 0)
		{
			return TextoCodigo(_codigo);
		}
		else
		{
			var texto:String = new String();
			for(var i:Number = 0; i < _combinacion.length; ++i)
			{
				texto = texto + TextoCodigo(_combinacion[i]) + " + ";
			}
			return texto + TextoCodigo(_codigo);
		}
	}

	//TextoCodigo()
	//Devuelve el String correspondiente a un codigo de Tecla
	public function TextoCodigo(codigo:Number):String
	{
		switch(codigo)
		{
			case 18: return "ALT";
			case 8: return "BACKSPACE";
			case 20: return "CAPS LOCK";
			case 17: return "CONTROL";
			case 46: return "DELETE";
			case 40: return "DOWN";
			case 35: return "END";
			case 13: return "ENTER";
			case 27: return "ESCAPE";
			case 36: return "HOME";
			case 45: return "INSERT";
			case 37: return "LEFT";
			case 34: return "PAGE DOWN";
			case 33: return "PAGE UP";
			case 39: return "RIGHT";
			case 16: return "SHIFT";
			case 32: return "SPACE";
			case 9: return "TAB";
			case 38: return "UP";
			case 96: return "NUMPAD 0";
			case 97: return "NUMPAD 1";
			case 98: return "NUMPAD 2";
			case 99: return "NUMPAD 3";
			case 100: return "NUMPAD 4";
			case 101: return "NUMPAD 5";
			case 102: return "NUMPAD 6";
			case 103: return "NUMPAD 7";
			case 104: return "NUMPAD 8";
			case 105: return "NUMPAD 9";
			case 112: return "F1";
			case 113: return "F2";
			case 114: return "F3";
			case 115: return "F4";
			case 116: return "F5";
			case 117: return "F6";
			case 118: return "F7";
			case 119: return "F8";
			case 120: return "F9";
			case 121: return "F10";
			case 122: return "F11";
			case 123: return "F12";
			default: return String.fromCharCode(codigo);
		}
	}

	//CodigoTexto()
	//Devuelve el codigo correspondiente a un String
	//Solo se definen los numeros de codigo mas comunes
	static function CodigoTexto(texto:String):Number
	{
		switch(texto)
		{
			case "ALT": return 18;
			case "BACKSPACE": return 8;
			case "CAPS LOCK": return 20;
			case "CONTROL": return 17;
			case "DELETE": return 46;
			case "DOWN": return 40;
			case "END": return 35;
			case "ENTER": return 13;
			case "ESCAPE": return 27;
			case "HOME": return 36;
			case "INSERT": return 45;
			case "LEFT": return 37;
			case "PAGE DOWN": return 34;
			case "PAGE UP": return 33;
			case "RIGHT": return 39;
			case "SHIFT": return 16;
			case "SPACE": return 32;
			case "TAB": return 9;
			case "UP": return 38;
			default: return 0;
		}
	}

}//Clase configuracion.datatypes.Tecla
